note
	description: "List with extra operations to move an item and to duplicate."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	MY_LIST [G]

create
	make

feature{NONE} -- Initialization

	make (a_size: INTEGER)
			-- Allocate list with `a_size' items.
		require
			valid_size: a_size > 0
		do
			create storage.make (a_size)
		end

feature -- MY_LIST Operation

	duplicate (n: INTEGER): MY_LIST [G]
			-- Copy of sub-list beginning at current position
			-- and having min (`n', count - index + 1) items.
		require
			not_off_unless_after: off implies after
			valid_count: n >= 0
		local
			idx: INTEGER
			to_be_copied, counter: INTEGER_32
		do
			from
				create Result.make (storage.count)
				idx := index
				to_be_copied := n.min (count - index + 1)
			until
				counter = to_be_copied
			loop
				Result.extend (item)
				forth
				counter := counter + 1
			end
			storage.go_i_th (idx)
		end

feature -- Basic list iteration

	start
			-- Start `storage'.
		do
			storage.start
		end

	forth
			-- Forth `storage'.
		require
			not after
		do
			storage.forth
		end

	after: BOOLEAN
			-- Is `storage' after?
		do
			Result := storage.after
		end

	item: G
			-- `item' of `storage'.
		require
			not off
		do
			Result := storage.item
		end

feature -- Basic list query

	has (a_item: G): BOOLEAN
			-- Has current `a_item' as a member?
		require
			a_item /= Void
		do
			Result := storage.has (a_item)
		end

	off: BOOLEAN
			-- Is `storage' off?
		do
			Result := storage.off
		end

	index: INTEGER
			-- `index' of `storage'.
		do
			Result := storage.index
		end

	count: INTEGER
			-- `count' of `storage'.
		do
			Result := storage.count
		end

feature -- Basic list command

	extend (a_item: G)
			-- Extend `a_item' to `storage'.
		require
			a_item /= Void
		do
			storage.extend (a_item)
		end

	remove
			-- Remove the item at `index'.
		require
			not off
		do
			storage.remove
		end

feature{NONE} -- Access

	storage: ARRAYED_LIST [G]
			-- Storage of current.

end
